<?php
/* ======================================================
 # Fix 404 Error Links for Joomla! - v2.2.7 (free version)
 # -------------------------------------------------------
 # For Joomla! CMS (v3.x)
 # Author: Web357 (Yiannis Christodoulou)
 # Copyright (©) 2014-2023 Web357. All rights reserved.
 # License: GNU/GPLv3, http://www.gnu.org/licenses/gpl-3.0.html
 # Website: https:/www.web357.com
 # Demo: https://demo.web357.com/joomla/fix-404-error-links
 # Support: support@web357.com
 # Last modified: Saturday 03 February 2024, 03:19:21 AM
 ========================================================= */
// No direct access to this file
defined('_JEXEC') or die;

use Joomla\Utilities\IpHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\MVC\View\HtmlView;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\AdministratorRouter;
use Joomla\CMS\HTML\Helpers\Sidebar;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Access\Exception\NotAllowed;
use Joomla\CMS\Application\CMSApplication;

class Fix404errorlinksViewError404logs extends HtmlView
{
	protected $items;
	protected $pagination;
	protected $state;

	function display($tpl = null) 
	{		
		// Get data from the model and then assign data to the view
		$this->state = $this->get('State');
		$this->items = $this->get('Items');
		$this->pagination = $this->get('Pagination');
		$this->filterForm = $this->get('FilterForm');
		$this->activeFilters = $this->get('ActiveFilters');
		$this->fixed_error404_urls = $this->getState('filter.fixed_error404_urls');

		$this->live_site = substr(Uri::root(), 0, -1);
		if (version_compare(JVERSION, '4.0', '>=')) {
			$this->router = Factory::getContainer()->get(AdministratorRouter::class);
		} else {
			$this->router = CMSApplication::getInstance('site')->getRouter('site');
		}

		// Get the correct IP address of the client
		$this->my_ip_address = IpHelper::getIp();
		if (!filter_var($this->my_ip_address, FILTER_VALIDATE_IP))
		{
			$this->my_ip_address = isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : '0.0.0.0';
		}
		$this->my_ip_address = '66.102.15.255';

		// get Model
       	$this->model = $this->getModel();

		// Check for model errors.
		if ($errors = $this->get('Errors'))
		{
			throw new NotAllowed(Text::_(implode('<br />', $errors)), 500);
		}

		if (version_compare(JVERSION, '3.0', 'ge'))
		{
			// J3X
			// Include helper submenu
			Fix404errorlinksHelper::addSubmenu('error404logs');
	
			// Show sidebar
			if (version_compare(JVERSION, '5.0', '>=')) {
				$this->sidebar = Sidebar::render();
			}
			else
			{
				$this->sidebar = JHtmlSidebar::render();
			}
		}

		// Set the toolbar
		$this->addToolBar();

		// Display the template
		parent::display($tpl);
	}

	protected function addToolBar()
	{
		$canDo = Fix404errorlinksHelper::getActions();

		// delete
		if($canDo->get('core.delete'))
		{
			ToolbarHelper::custom('error404logs.deletelogs', 'delete.png', 'delete_f2.png', 'JTOOLBAR_DELETE', true);
			ToolbarHelper::custom('error404logs.deleteAllLogs', 'purge.png', 'purge_f2.png', 'COM_FIX404ERRORLINKS_DELETE_ALL_404_ERROR_LINKS', false);
			ToolbarHelper::custom('error404logs.deleteFixed404ErrorURLs', 'ok.png', 'ok_f2.png', 'COM_FIX404ERRORLINKS_DELETE_ALL_404_FIXED_ERROR_LINKS', false);
		}

		ToolbarHelper::link(Route::_('index.php?option=com_fix404errorlinks'), 'COM_FIX404ERRORLINKS_CPANEL', 'arrow-left');

		// parameters
		if($canDo->get('core.admin'))
		{
			ToolbarHelper::divider();
			ToolbarHelper::preferences('com_fix404errorlinks');
		}

		// Set ToolBar title
		ToolbarHelper::title(Text::_('COM_FIX404ERRORLINKS_ERROR_404_LINKS'), 'link.png');
	}
	
	protected function getSortFields()
	{
		return array(
		'a.id' => Text::_('JGRID_HEADING_ID'),
		'a.error_url' => Text::_('COM_FIX404ERRORLINKS_ERROR_URL'),
		'a.hits' => Text::_('COM_FIX404ERRORLINKS_HITS'),
		);
	}

    /**
     * Check if state is set
     *
     * @param   mixed  $state  State
     *
     * @return bool
     */
    public function getState($state)
    {
        return isset($this->state->{$state}) ? $this->state->{$state} : false;
    }
}