<?php
/* ======================================================
 # Fix 404 Error Links for Joomla! - v2.2.7 (free version)
 # -------------------------------------------------------
 # For Joomla! CMS (v4.x)
 # Author: Web357 (Yiannis Christodoulou)
 # Copyright (©) 2014-2023 Web357. All rights reserved.
 # License: GNU/GPLv3, http://www.gnu.org/licenses/gpl-3.0.html
 # Website: https:/www.web357.com
 # Demo: https://demo.web357.com/joomla/fix-404-error-links
 # Support: support@web357.com
 # Last modified: Saturday 03 February 2024, 03:19:21 AM
 ========================================================= */
// No direct access
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\MVC\View\HtmlView;
use Joomla\CMS\Router\AdministratorRouter;
use Joomla\CMS\HTML\Helpers\Sidebar;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Application\CMSApplication;
/**
 * View class for a list of Fix404errorlinks.
 *
 * @since  1.6
 */
class Fix404errorlinksViewRedirects extends HtmlView
{
	protected $items;

	protected $pagination;

	protected $state;

	/**
	 * Display the view
	 *
	 * @param   string  $tpl  Template name
	 *
	 * @return void
	 *
	 * @throws Exception
	 */
	public function display($tpl = null)
	{
		$this->state = $this->get('State');
		$this->items = $this->get('Items');
		$this->pagination = $this->get('Pagination');
        $this->filterForm = $this->get('FilterForm');
		$this->activeFilters = $this->get('ActiveFilters');

		$this->live_site = substr(Uri::root(), 0, -1);

		if (version_compare(JVERSION, '4.0', '>=')) {
			$this->router = Factory::getContainer()->get(AdministratorRouter::class);
		} else {
			$this->router = CMSApplication::getInstance('site')->getRouter('site');
		}
		
		// Check for errors.
		if (count($errors = $this->get('Errors')))
		{
			throw new Exception(implode("\n", $errors));
		}

		Fix404errorlinksHelper::addSubmenu('redirects');

		$this->addToolbar();

		// Show sidebar
		if (version_compare(JVERSION, '5.0', '>=')) {
			$this->sidebar = Sidebar::render();
		}
		else
		{
			$this->sidebar = JHtmlSidebar::render();
		}
		parent::display($tpl);
	}

	/**
	 * Add the page title and toolbar.
	 *
	 * @return void
	 *
	 * @since    1.6
	 */
	protected function addToolbar()
	{
		$state = $this->get('State');
		$canDo = Fix404errorlinksHelper::getActions();

		ToolbarHelper::title(Text::_('COM_FIX404ERRORLINKS_REDIRECTS'), 'link.png');

		// Check if the form exists before showing the add/edit buttons
		$formPath = JPATH_COMPONENT_ADMINISTRATOR . '/views/redirect';

		if (file_exists($formPath))
		{
			if ($canDo->get('core.create'))
			{
				ToolbarHelper::addNew('redirect.add', 'JTOOLBAR_NEW');

				if (isset($this->items[0]))
				{
					ToolbarHelper::custom('redirects.duplicate', 'copy.png', 'copy_f2.png', 'JTOOLBAR_DUPLICATE', true);
				}
			}

			if ($canDo->get('core.edit') && isset($this->items[0]))
			{
				ToolbarHelper::editList('redirect.edit', 'JTOOLBAR_EDIT');
			}
		}

		if ($canDo->get('core.edit.state'))
		{
			if (isset($this->items[0]->state))
			{
				ToolbarHelper::divider();
				ToolbarHelper::custom('redirects.publish', 'publish.png', 'publish_f2.png', 'JTOOLBAR_PUBLISH', true);
				ToolbarHelper::custom('redirects.unpublish', 'unpublish.png', 'unpublish_f2.png', 'JTOOLBAR_UNPUBLISH', true);
			}
			elseif (isset($this->items[0]))
			{
				// If this component does not use state then show a direct delete button as we can not trash
				ToolbarHelper::deleteList('', 'redirects.delete', 'JTOOLBAR_DELETE');
			}

			if (isset($this->items[0]->state))
			{
				ToolbarHelper::divider();
				ToolbarHelper::archiveList('redirects.archive', 'JTOOLBAR_ARCHIVE');
			}

			if (isset($this->items[0]->checked_out))
			{
				ToolbarHelper::custom('redirects.checkin', 'checkin.png', 'checkin_f2.png', 'JTOOLBAR_CHECKIN', true);
			}
		}

		// Show trash and delete for components that uses the state field
		if (isset($this->items[0]->state))
		{
			if ($state->get('filter.state') == -2 && $canDo->get('core.delete'))
			{
				ToolbarHelper::deleteList('', 'redirects.delete', 'JTOOLBAR_EMPTY_TRASH');
				ToolbarHelper::divider();
			}
			elseif ($canDo->get('core.edit.state'))
			{
				ToolbarHelper::trash('redirects.trash', 'JTOOLBAR_TRASH');
				ToolbarHelper::divider();
			}
		}

		// import from "redirect" joomla! core component
		if ($canDo->get('core.create'))
		{
			ToolbarHelper::custom('redirects.comRedirectImport', 'upload.png', 'upload_f2.png', 'COM_FIX404ERRORLINKS_COMREDIRECT_IMPORT', false);
		}

		if ($canDo->get('core.admin'))
		{
			ToolbarHelper::preferences('com_fix404errorlinks');
		}

		if (version_compare(JVERSION, '5.0', '>=')) {
			Sidebar::setAction('index.php?option=com_fix404errorlinks&view=redirects');
		}
		else
		{
			JHtmlSidebar::setAction('index.php?option=com_fix404errorlinks&view=redirects');
		}
	}

	/**
	 * Method to order fields 
	 *
	 * @return void 
	 */
	protected function getSortFields()
	{
		return array(
			'a.id' => Text::_('JGRID_HEADING_ID'),
			'a.ordering' => Text::_('JGRID_HEADING_ORDERING'),
			'a.state' => Text::_('JSTATUS'),
			'a.source_url' => Text::_('COM_FIX404ERRORLINKS_REDIRECTIONS_SOURCE_URL'),
			'a.target_url' => Text::_('COM_FIX404ERRORLINKS_REDIRECTIONS_TARGET_URL'),
			'a.hits' => Text::_('COM_FIX404ERRORLINKS_REDIRECTIONS_HITS'),
			'a.last_access' => Text::_('COM_FIX404ERRORLINKS_REDIRECTIONS_LAST_ACCESS'),
			'a.action_type' => Text::_('COM_FIX404ERRORLINKS_REDIRECTIONS_ACTION_TYPE'),
			'a.action_code' => Text::_('COM_FIX404ERRORLINKS_REDIRECTIONS_ACTION_CODE'),
		);
	}

    /**
     * Check if state is set
     *
     * @param   mixed  $state  State
     *
     * @return bool
     */
    public function getState($state)
    {
        return isset($this->state->{$state}) ? $this->state->{$state} : false;
    }
}
